<?php

/**
 * @file
 * Contains \Drupal\module_missing_message_fixer\ModuleMissingMessageFixer.
 */

namespace Drupal\module_missing_message_fixer;

class ModuleMissingMessageFixer {

  /**
   * Set the table headers for the ui and drush.
   *
   * @return string[]
   *   Format: $[$column_key] = $cell
   */
  public static function getTableHeader() {
    return array(
      'name' => 'Name',
      'type' => 'Type',
      'filename' => 'Filename',
    );
  }


  /**
   * Produces one table row for each missing module.
   *
   * The table rows are suitable for drush and for the admin UI.
   *
   * @return array[]
   *   Format: $[$extension_name][$column_key] = $cell
   */
  function getTableRows() {
    // Initalize vars.
    $rows = [];
    $i = 0;
    $db = \Drupal::database();

    // Grab all the modules in the system table.
    $results = $db->select('key_value', 'k')
      ->fields('k', ['name'])
      ->condition('collection', 'system.schema')
      ->execute()
      ->fetchAllKeyed();

    // Grab all the modules the system can see.
    $modules = \Drupal::state()->get('system.module.files', []);

    // Get the diff then fill the array with calues.
    // array_fill didn't seem to work here.
    // So I am using the foreach.
    $diff = array_diff_key($results, $modules);
    foreach ($diff as &$value) {
      $value = $i;
      $i++;
    }

    // Now flip this.
    $records = array_flip($diff);

    // Go through the query and check the missing modules.
    // Plus do our checks to see what's wrong.
    foreach ($records as $record) {

      if ($record === 'default') {
        continue;
      }

      // Grab the checker.
      $filename = drupal_get_filename('module', $record);

      if ($filename === NULL) {
        // Report this module in the table.
        $rows[$record] = array(
          'name' => $record,
          'type' => 'module',
          'filename' => '',
        );
        continue;
      }

      /**
       * @todo  need to figure out how to grab the file name from the module
       * since it is not in the new key_value table.
       * If it is missing from the \Drupal::state()->get('system.module.files
       * Then it has to be somewhere in cache or something.
       */

      // $message = NULL;
      // $replacements = array(
      //   '@name' => $record->name,
      //   '@type' => $record->type,
      //   '@file' => $filename,
      // );
      // if (!file_exists($filename)) {
      //   // This case is unexpected, because drupal_get_filename() should take care
      //   // of it already.
      //   $message = 'The file @file for @name @type is missing.';
      // }
      // elseif (!is_readable($filename)) {
      //   // This case is unexpected, because drupal_get_filename() should take care
      //   // of it already.
      //   $message = 'The file @file for @name @type is not readable.';
      // }
      // else {
      //   // Verify if *.info file exists.
      //   // See https://www.drupal.org/node/2789993#comment-12306555
      //   $info_filename = dirname($filename) . '/' . $record->name . '.info.yml';
      //   $replacements['@info_file'] = $info_filename;
      //   if (!file_exists($info_filename)) {
      //     $message = 'The *.info.yml file @info_file for @name @type is missing.';

      //   }
      //   elseif (!is_readable($info_filename)) {
      //     $message = 'The *.info.yml file @info_file for @name @type is not readable.';
      //   }
      // }

      // if ($message !== NULL) {
      //   // This case should never occur.
      //   drupal_set_message(
      //     t($message, $replacements),
      //     'warning',
      //     FALSE);
      // }
    }

    return $rows;
  }
}
